/*
  Projekt: Network Driver for NIC 3c509
           TUOX Projekt
  Autori:  Miroslav Kubiczek
           Martin Fanta

  From linux driver!
*/
#include <time.h>  // variable type time_t

/*--------------------------------------------------------------------*/
/*
  System dependent structures and functions
*/
int el3_probe(struct device *dev);

struct device {
  uchar dev_addr[6];
  ushort irq, base_addr, if_port, inter, tbusy, start, flags;
  char *name;
  void *priv;
  int   (*open)(struct device *);
  void (*t_interrupt)(struct device *);
  int	(*hard_start_xmit)(struct device *dev, uchar *data, uint len);
  int	(*close)(struct device *);

  time_t trans_start;
};
/*--------------------------------------------------------------------*/
/* To minimize the size of the driver source I only define operating
   constants if they are used several times.  You'll need the manual
   if you want to understand driver details.

   We did not do it!
*/

/*
  3com NIC adapter consits of several 8-word register windows.
  Windows' definition:
  0 - Setup
  1 - Operating Set
  2 - Station Address Setup/Read
  3 - FIFO Management
  4 - Diagnostic
  5 - Command Results and Internal State
  6 - Statistics

  The main registers:
    Command Register
      in all windows/Port 0Eh (=EL3_CMD). Write only
    Status Register
      in all windows/Port 0Eh (=EL3_CMD). Read only
*/
#define EL3WINDOW(win_num) outw(SelectWindow + (win_num), ioaddr + EL3_CMD)

/* Offsets from base I/O address. */
#define EL3_DATA 0x00
#define EL3_CMD 0x0e
#define EL3_STATUS 0x0e
#define	EEPROM_READ 0x80

#define EL3_IO_EXTENT	16

/* The top five bits written to EL3_CMD are a command,
   the lower 11 bits are the parameter, if applicable. */
enum c509cmd {
	TotalReset = 0<<11,       /* globalni nulovani */
  SelectWindow = 1<<11,     /* vyber okna */
  StartCoax = 2<<11,
	RxDisable = 3<<11,        /* zakaz prijem */
  RxEnable = 4<<11,         /* povol prijem */
  RxReset = 5<<11,          /* resetuj prijem - nasleduje RxEnable
                                                pro pokracovani v prijmu */
  RxDiscard = 8<<11,        /* odstran prvni prijmuty paket z FIFO */
	TxEnable = 9<<11,         /* povol odesilani */
  TxDisable = 10<<11,       /* zakaz odesilani */
  TxReset = 11<<11,         /* resetuj odesilani - nasleduje RxReset
                                                   pro dalsi odesilani */
	FakeIntr = 12<<11,        /* nastavenim vyzaduj generovani
                               sveho preruseni */
  AckIntr = 13<<11,         /* potvrd obsluhu preruseni */
  SetIntrEnb = 14<<11,      /* nastav masku pro preruseni */
	SetStatusEnb = 15<<11,    /* nastav masku pro registr stavu
                               0 v danem bitu - dany stav se neukazuje, je 0 */
  SetRxFilter = 16<<11,     /* nastav filtrovani adres  */
  SetRxThreshold = 17<<11,  /* nastav pocet prijmutych bytu pro vyvolani
                               preruseni */
	SetTxThreshold = 18<<11,  /* nastav pocet volnych bytu v FIFO pro
                               vyvolani preruseni */
  SetTxStart = 19<<11,      /* nastav pocet bytu z paketu v FIFO pro
                               zacatek vysilani */
  StatsEnable = 21<<11,     /* povol statistiku komunikace */
	StatsDisable = 22<<11,    /* zakaz statistiku komunikace */
  StopCoax = 23<<11,
};

/*
  Bity registru stavu
*/
enum c509status {
	IntLatch = 0x0001,        /* indikace pozadavku a vyvolani preruseni */
  AdapterFailure = 0x0002,  /* indikace preruseni ...*/
  TxComplete = 0x0004,      /* ... */
	TxAvailable = 0x0008,     /* ... */
  RxComplete = 0x0010,      /* ... */
  RxEarly = 0x0020,         /* ... */
	IntReq = 0x0040,          /* ... sam jsem ho vyvolal */
  StatsFull = 0x0080,       /* ... statisticke registry jsou preplnene */
  CmdBusy = 0x1000,         /* prikaz se stale provadi,
                               pouze pro nektere casove narocnejsi prikazy */
};

/* The SetRxFilter command accepts the following classes: */
enum RxFilter {
	RxStation = 1,
  RxMulticast = 2,
  RxBroadcast = 4,
  RxProm = 8            /* All addreses (promiscous mode) */
};

/* Register window 1 offsets for registers.
   The window is used in normal operation. */
#define TX_FIFO		 0x00
#define RX_FIFO		 0x00
#define RX_STATUS  0x08
#define TX_STATUS  0x0B
#define TX_FREE		 0x0C		/* Remaining free bytes in Tx buffer. */

#define WN0_IRQ		 0x08		/* Window 0: Set IRQ line in bits 12-15. */
#define WN4_MEDIA	 0x0A		/* Window 4: Various transcvr/media bits. */
#define MEDIA_TP	 0x00C0		/* Enable link beat and jabber for 10baseT. */
